@@ -17,7 +17,17 @@ from os.path import expanduser
 from threading import Lock
 from threading import Thread
 
-from .compat import (TO_UNICODE, urlencode)
+import keyring
+
+from .compat import (TO_UNICODE, urlencode, IS_LINUX)
+from .constants import (
+    HTTP_HEADER_CONTENT_TYPE,
+    HTTP_HEADER_ACCEPT,
+    HTTP_HEADER_USER_AGENT,
+    HTTP_HEADER_SERVICE_NAME,
+    PARAMETER_CLIENT_STORE_TEMPORARY_CREDENTIAL,
+    PARAMETER_CLIENT_USE_SECURE_STORAGE_FOR_TEMPORARY_CREDENTIAL,
+)
 from .errorcode import (ER_FAILED_TO_CONNECT_TO_DB, ER_INVALID_VALUE)
 from .errors import (Error,
                      DatabaseError,
@@ -52,7 +62,7 @@ if not path.exists(CACHE_DIR):
     try:
         makedirs(CACHE_DIR, mode=0o700)
     except Exception as ex:
-        logger.warning('cannot create a cache directory: [%s], err=[%s]',
+        logger.debug('cannot create a cache directory: [%s], err=[%s]',
                        CACHE_DIR, ex)
         CACHE_DIR = None
 logger.debug("cache directory: %s", CACHE_DIR)
@@ -70,6 +80,10 @@ TEMPORARY_CREDENTIAL_FILE = path.join(
 # temporary credential cache lock directory name
 TEMPORARY_CREDENTIAL_FILE_LOCK = TEMPORARY_CREDENTIAL_FILE + ".lck"
 
+# keyring
+KEYRING_SERVICE_NAME = "net.snowflake.temporary_token"
+KEYRING_USER = "temp_token"
+
 
 class AuthByPlugin(object):
     """
@@ -83,7 +97,8 @@ class AuthByPlugin(object):
     def update_body(self, body):
         raise NotImplementedError
 
-    def authenticate(self, authenticator, account, user, password):
+    def authenticate(
+            self, authenticator, service_name, account, user, password):
         raise NotImplementedError
 
     def handle_failure(self, ret):
@@ -96,14 +111,9 @@ class AuthByPlugin(object):
             self._rest._connection, None, DatabaseError,
             {
                 u'msg': (u"Failed to connect to DB: {host}:{port}, "
-                         u"proxies={proxy_host}:{proxy_port}, "
-                         u"proxy_user={proxy_user}, "
                          u"{message}").format(
                     host=self._rest._host,
                     port=self._rest._port,
-                    proxy_host=self._rest._proxy_host,
-                    proxy_port=self._rest._proxy_port,
-                    proxy_user=self._rest._proxy_user,
                     message=ret[u'message'],
                 ),
                 u'errno': int(ret.get(u'code', -1)),
@@ -154,10 +164,13 @@ class Auth(object):
 
         request_id = TO_UNICODE(uuid.uuid4())
         headers = {
-            u'Content-Type': CONTENT_TYPE_APPLICATION_JSON,
-            u"accept": ACCEPT_TYPE_APPLICATION_SNOWFLAKE,
-            u"User-Agent": PYTHON_CONNECTOR_USER_AGENT,
+            HTTP_HEADER_CONTENT_TYPE: CONTENT_TYPE_APPLICATION_JSON,
+            HTTP_HEADER_ACCEPT: ACCEPT_TYPE_APPLICATION_SNOWFLAKE,
+            HTTP_HEADER_USER_AGENT: PYTHON_CONNECTOR_USER_AGENT,
         }
+        if HTTP_HEADER_SERVICE_NAME in session_parameters:
+            headers[HTTP_HEADER_SERVICE_NAME] = \
+                session_parameters[HTTP_HEADER_SERVICE_NAME]
         url = u"/session/v1/login-request"
         body_template = Auth.base_auth_data(
             user, account, self._rest._connection.application,
@@ -216,14 +229,10 @@ class Auth(object):
             # HTTP 403
             raise err.__class__(
                 msg=(u"Failed to connect to DB. "
-                     u"Verify the account name is correct: {host}:{port}, "
-                     u"proxies={proxy_host}:{proxy_port}, "
-                     u"proxy_user={proxy_user}. {message}").format(
+                     u"Verify the account name is correct: {host}:{port}. "
+                     u"{message}").format(
                     host=self._rest._host,
                     port=self._rest._port,
-                    proxy_host=self._rest._proxy_host,
-                    proxy_port=self._rest._proxy_port,
-                    proxy_user=self._rest._proxy_user,
                     message=TO_UNICODE(err)
                 ),
                 errno=ER_FAILED_TO_CONNECT_TO_DB,
@@ -232,14 +241,10 @@ class Auth(object):
             # HTTP 502/504
             raise err.__class__(
                 msg=(u"Failed to connect to DB. "
-                     u"Service is unavailable: {host}:{port}, "
-                     u"proxies={proxy_host}:{proxy_port}, "
-                     u"proxy_user={proxy_user}. {message}").format(
+                     u"Service is unavailable: {host}:{port}. "
+                     u"{message}").format(
                     host=self._rest._host,
                     port=self._rest._port,
-                    proxy_host=self._rest._proxy_host,
-                    proxy_port=self._rest._proxy_port,
-                    proxy_user=self._rest._proxy_user,
                     message=TO_UNICODE(err)
                 ),
                 errno=ER_FAILED_TO_CONNECT_TO_DB,
@@ -285,15 +290,9 @@ class Auth(object):
                     {
                         u'msg': (u"Failed to connect to DB. MFA "
                                  u"authentication failed: {"
-                                 u"host}:{port}, "
-                                 u"proxies={proxy_host}:{proxy_port}, "
-                                 u"proxy_user={proxy_user}, "
-                                 u"{message}").format(
+                                 u"host}:{port}. {message}").format(
                             host=self._rest._host,
                             port=self._rest._port,
-                            proxy_host=self._rest._proxy_host,
-                            proxy_port=self._rest._proxy_port,
-                            proxy_user=self._rest._proxy_user,
                             message=ret[u'message'],
                         ),
                         u'errno': ER_FAILED_TO_CONNECT_TO_DB,
@@ -321,15 +320,10 @@ class Auth(object):
             Error.errorhandler_wrapper(
                 self._rest._connection, None, DatabaseError,
                 {
-                    u'msg': (u"Failed to connect to DB: {host}:{port}, "
-                             u"proxies={proxy_host}:{proxy_port}, "
-                             u"proxy_user={proxy_user}, "
+                    u'msg': (u"Failed to connect to DB: {host}:{port}. "
                              u"{message}").format(
                         host=self._rest._host,
                         port=self._rest._port,
-                        proxy_host=self._rest._proxy_host,
-                        proxy_port=self._rest._proxy_port,
-                        proxy_user=self._rest._proxy_user,
                         message=ret[u'message'],
                     ),
                     u'errno': ER_FAILED_TO_CONNECT_TO_DB,
@@ -355,7 +349,9 @@ class Auth(object):
                 id_token_password=ret[u'data'].get(u'idTokenPassword'))
             if self._rest._connection.consent_cache_id_token:
                 write_temporary_credential_file(
-                    account, user, self._rest.id_token)
+                    account, user, self._rest.id_token,
+                    session_parameters.get(
+                        PARAMETER_CLIENT_USE_SECURE_STORAGE_FOR_TEMPORARY_CREDENTIAL))
             if u'sessionId' in ret[u'data']:
                 self._rest._connection._session_id = ret[u'data'][u'sessionId']
             if u'sessionInfo' in ret[u'data']:
@@ -414,8 +410,11 @@ class Auth(object):
                 })
 
     def read_temporary_credential(self, account, user, session_parameters):
-        if session_parameters.get('CLIENT_STORE_TEMPORARY_CREDENTIAL'):
-            read_temporary_credential_file()
+        if session_parameters.get(PARAMETER_CLIENT_STORE_TEMPORARY_CREDENTIAL):
+            read_temporary_credential_file(
+                session_parameters.get(
+                    PARAMETER_CLIENT_USE_SECURE_STORAGE_FOR_TEMPORARY_CREDENTIAL)
+            )
             id_token = TEMPORARY_CREDENTIAL.get(
                 account.upper(), {}).get(user.upper())
             if id_token:
@@ -431,7 +430,9 @@ class Auth(object):
         return False
 
 
-def write_temporary_credential_file(account, user, id_token):
+def write_temporary_credential_file(
+        account, user, id_token,
+        use_secure_storage_for_temporary_credential=False):
     if not CACHE_DIR or not id_token:
         # no cache is enabled or no id_token is given
         return
@@ -448,13 +449,19 @@ def write_temporary_credential_file(account, user, id_token):
                 break
             time.sleep(1)
         else:
-            logger.warning("The lock file still persists. Will ignore and "
+            logger.debug("The lock file still persists. Will ignore and "
                            "write the temporary credential file: %s",
                            TEMPORARY_CREDENTIAL_FILE)
         try:
-            with codecs.open(TEMPORARY_CREDENTIAL_FILE, 'w',
-                             encoding='utf-8', errors='ignore') as f:
-                json.dump(TEMPORARY_CREDENTIAL, f)
+            if IS_LINUX or not use_secure_storage_for_temporary_credential:
+                with codecs.open(TEMPORARY_CREDENTIAL_FILE, 'w',
+                                 encoding='utf-8', errors='ignore') as f:
+                    json.dump(TEMPORARY_CREDENTIAL, f)
+            else:
+                keyring.set_password(
+                    KEYRING_SERVICE_NAME, KEYRING_USER,
+                    json.dumps(TEMPORARY_CREDENTIAL))
+
         except Exception as ex:
             logger.debug("Failed to write a credential file: "
                          "file=[%s], err=[%s]", TEMPORARY_CREDENTIAL_FILE, ex)
@@ -462,7 +469,8 @@ def write_temporary_credential_file(account, user, id_token):
             unlock_temporary_credential_file()
 
 
-def read_temporary_credential_file():
+def read_temporary_credential_file(
+        use_secure_storage_for_temporary_credential=False):
     """
     Read temporary credential file
     """
@@ -479,14 +487,18 @@ def read_temporary_credential_file():
                 break
             time.sleep(1)
         else:
-            logger.warning("The lock file still persists. Will ignore and "
+            logger.debug("The lock file still persists. Will ignore and "
                            "write the temporary credential file: %s",
                            TEMPORARY_CREDENTIAL_FILE)
         try:
-            with codecs.open(
-                    TEMPORARY_CREDENTIAL_FILE, 'r',
-                    encoding='utf-8', errors='ignore') as f:
-                TEMPORARY_CREDENTIAL = json.load(f)
+            if IS_LINUX or not use_secure_storage_for_temporary_credential:
+                with codecs.open(TEMPORARY_CREDENTIAL_FILE, 'r',
+                                 encoding='utf-8', errors='ignore') as f:
+                    TEMPORARY_CREDENTIAL = json.load(f)
+            else:
+                f = keyring.get_password(
+                    KEYRING_SERVICE_NAME, KEYRING_USER) or "{}"
+                TEMPORARY_CREDENTIAL = json.loads(f)
             return TEMPORARY_CREDENTIAL
         except Exception as ex:
             logger.debug("Failed to read a credential file. The file may not"
@@ -518,16 +530,24 @@ def unlock_temporary_credential_file():
         return False
 
 
-def delete_temporary_credential_file():
+def delete_temporary_credential_file(
+        use_secure_storage_for_temporary_credential=False):
     """
     Delete temporary credential file and its lock file
     """
     global TEMPORARY_CREDENTIAL_FILE
-    try:
-        remove(TEMPORARY_CREDENTIAL_FILE)
-    except Exception as ex:
-        logger.debug("Failed to delete a credential file: "
-                     "file=[%s], err=[%s]", TEMPORARY_CREDENTIAL_FILE, ex)
+    if IS_LINUX or not use_secure_storage_for_temporary_credential:
+        try:
+            remove(TEMPORARY_CREDENTIAL_FILE)
+        except Exception as ex:
+            logger.debug("Failed to delete a credential file: "
+                         "file=[%s], err=[%s]", TEMPORARY_CREDENTIAL_FILE, ex)
+    else:
+        try:
+            keyring.delete_password(KEYRING_SERVICE_NAME, KEYRING_USER)
+        except Exception as ex:
+            logger.debug("Failed to delete credential in the keyring: err=[%s]",
+                         ex)
     try:
         removedirs(TEMPORARY_CREDENTIAL_FILE_LOCK)
     except Exception as ex:
