@@ -60,7 +60,7 @@ class SnowflakeCursor(object):
     PUT_SQL_RE = re.compile(u(r'^(?:/\*.*\*/\s*)*put\s+'), flags=re.IGNORECASE)
     GET_SQL_RE = re.compile(u(r'^(?:/\*.*\*/\s*)*get\s+'), flags=re.IGNORECASE)
     INSERT_SQL_RE = re.compile(u(r'^insert\s+into'), flags=re.IGNORECASE)
-    COMMENT_SQL_RE = re.compile(u"/\*.*\*/")
+    COMMENT_SQL_RE = re.compile(r"/\*.*\*/")
     INSERT_SQL_VALUES_RE = re.compile(u(r'.*VALUES\s*(\(.*\)).*'),
                                       re.IGNORECASE | re.MULTILINE | re.DOTALL)
     ALTER_SESSION_RE = re.compile(
@@ -416,9 +416,12 @@ class SnowflakeCursor(object):
     def execute(self, command, params=None, timeout=None,
                 _do_reset=True,
                 _put_callback=None,
+                _put_azure_callback=None,
                 _put_callback_output_stream=sys.stdout,
                 _get_callback=None,
+                _get_azure_callback=None,
                 _get_callback_output_stream=sys.stdout,
+                _show_progress_bar=True,
                 _statement_params=None,
                 _is_internal=False,
                 _no_results=False,
@@ -466,7 +469,7 @@ class SnowflakeCursor(object):
         if m:
             query1 = u'describe table {0}'.format(m.group(1))
             if logger.getEffectiveLevel() <= logging.WARNING:
-                logger.warning(
+                logger.info(
                     u'query was rewritten: org=%s, new=%s',
                     u' '.join(line.strip() for line in query.split(u'\n')),
                     query1
@@ -512,9 +515,12 @@ class SnowflakeCursor(object):
                 sf_file_transfer_agent = SnowflakeFileTransferAgent(
                     self, query, ret,
                     put_callback=_put_callback,
+                    put_azure_callback=_put_azure_callback,
                     put_callback_output_stream=_put_callback_output_stream,
                     get_callback=_get_callback,
+                    get_azure_callback=_get_azure_callback,
                     get_callback_output_stream=_get_callback_output_stream,
+                    show_progress_bar=_show_progress_bar,
                     raise_put_get_error=_raise_put_get_error)
                 sf_file_transfer_agent.execute()
                 data = sf_file_transfer_agent.result()
